/* MIDI Writing library.  Written by David Madore <david.madore@ens.fr> */

/* Public Domain */

/* $Id: midiguile.c,v 1.1 2000/07/21 01:37:02 david Exp $ */

#include <libguile.h>
#include "midiwrite.h"

static long SMOB_midi_tag;

static char NAME_midi_init[] = "midi-init";
static char NAME_midi_wait[] = "midi-wait";
static char NAME_midi_sync[] = "midi-sync";
static char NAME_midi_all_wait[] = "midi-all-wait";
static char NAME_midi_prg[] = "midi-prg";
static char NAME_midi_ctl[] = "midi-ctl";
static char NAME_midi_n_on[] = "midi-n-on";
static char NAME_midi_n_off[] = "midi-n-off";
static char NAME_midi_pitchbend[] = "midi-pitchbend";
static char NAME_midi_set_subdiv[] = "midi-set-subdiv";
static char NAME_midi_meta_tempo[] = "midi-meta-tempo";
static char NAME_midi_meta_beat[] = "midi-meta-beat";
static char NAME_midi_meta_key[] = "midi-meta-key";
static char NAME_midi_meta_text[] = "midi-meta-text";
static char NAME_midi_meta_copyright[] = "midi-meta-copyright";
static char NAME_midi_meta_sequence[] = "midi-meta-sequence";
static char NAME_midi_meta_trackname[] = "midi-meta-trackname";
static char NAME_midi_meta_instrument[] = "midi-meta-instrument";
static char NAME_midi_meta_lyrics[] = "midi-meta-lyrics";
static char NAME_midi_meta_marker[] = "midi-meta-marker";
static char NAME_midi_meta_cuepoint[] = "midi-meta-cuepoint";
static char NAME_midi_single_note[] = "midi-single-note";
static char NAME_midi_start[] = "midi-start";
static char NAME_midi_finish[] = "midi-finish";
static char NAME_midi_dump[] = "midi-dump";

static SCM
GUILE_midi_init (SCM GUILE_nbvoices_)
{
  int nbvoices_;
  midi_ctx m;
  SCM GUILE_m;

  nbvoices_ = scm_num2long (GUILE_nbvoices_, (char *)SCM_ARG1,
			    NAME_midi_init);
  m = midi_init (nbvoices_);
  SCM_NEWCELL (GUILE_m);
  SCM_SETCDR (GUILE_m, m);
  SCM_SETCAR (GUILE_m, SMOB_midi_tag);
  return GUILE_m;
}

static SCM
GUILE_midi_wait (SCM GUILE_m, SCM GUILE_trk, SCM GUILE_ln)
{
  midi_ctx m;
  int trk, ln;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_wait);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  trk = scm_num2long (GUILE_trk, (char *)SCM_ARG2,
		      NAME_midi_wait);
  ln = scm_num2long (GUILE_ln, (char *)SCM_ARG3,
		     NAME_midi_wait);
  midi_wait (m, trk, ln);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_sync (SCM GUILE_m)
{
  midi_ctx m;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_sync);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  midi_sync (m);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_all_wait (SCM GUILE_m, SCM GUILE_ln)
{
  midi_ctx m;
  int ln;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_all_wait);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  ln = scm_num2long (GUILE_ln, (char *)SCM_ARG2,
		     NAME_midi_all_wait);
  midi_all_wait (m, ln);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_prg (SCM GUILE_m, SCM GUILE_trk, SCM GUILE_ch, SCM GUILE_prg)
{
  midi_ctx m;
  int trk;
  char ch;
  int prg;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_prg);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  trk = scm_num2long (GUILE_trk, (char *)SCM_ARG2,
		      NAME_midi_prg);
  ch = scm_num2long (GUILE_ch, (char *)SCM_ARG3,
		     NAME_midi_prg);
  prg = scm_num2long (GUILE_prg, (char *)SCM_ARG4,
		      NAME_midi_prg);
  midi_prg (m, trk, ch, prg);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_ctl (SCM GUILE_m, SCM GUILE_trk, SCM GUILE_ch, SCM GUILE_ctl, SCM GUILE_val)
{
  midi_ctx m;
  int trk;
  char ch, ctl, val;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_ctl);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  trk = scm_num2long (GUILE_trk, (char *)SCM_ARG2,
		      NAME_midi_ctl);
  ch = scm_num2long (GUILE_ch, (char *)SCM_ARG3,
		     NAME_midi_ctl);
  ctl = scm_num2long (GUILE_ctl, (char *)SCM_ARG4,
		      NAME_midi_ctl);
  val = scm_num2long (GUILE_val, (char *)SCM_ARG5,
		      NAME_midi_ctl);
  midi_ctl (m, trk, ch, ctl, val);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_n_on (SCM GUILE_m, SCM GUILE_trk, SCM GUILE_ch, SCM GUILE_nt, SCM GUILE_sp)
{
  midi_ctx m;
  int trk;
  char ch, nt, sp;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_n_on);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  trk = scm_num2long (GUILE_trk, (char *)SCM_ARG2,
		      NAME_midi_n_on);
  ch = scm_num2long (GUILE_ch, (char *)SCM_ARG3,
		     NAME_midi_n_on);
  nt = scm_num2long (GUILE_nt, (char *)SCM_ARG4,
		     NAME_midi_n_on);
  sp = scm_num2long (GUILE_sp, (char *)SCM_ARG5,
		     NAME_midi_n_on);
  midi_n_on (m, trk, ch, nt, sp);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_n_off (SCM GUILE_m, SCM GUILE_trk, SCM GUILE_ch, SCM GUILE_nt, SCM GUILE_sp)
{
  midi_ctx m;
  int trk;
  char ch, nt, sp;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_n_off);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  trk = scm_num2long (GUILE_trk, (char *)SCM_ARG2,
		      NAME_midi_n_off);
  ch = scm_num2long (GUILE_ch, (char *)SCM_ARG3,
		     NAME_midi_n_off);
  nt = scm_num2long (GUILE_nt, (char *)SCM_ARG4,
		     NAME_midi_n_off);
  sp = scm_num2long (GUILE_sp, (char *)SCM_ARG5,
		     NAME_midi_n_off);
  midi_n_off (m, trk, ch, nt, sp);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_pitchbend (SCM GUILE_m, SCM GUILE_trk, SCM GUILE_ch, SCM GUILE_val)
{
  midi_ctx m;
  int trk;
  char ch;
  short int val;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_pitchbend);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  trk = scm_num2long (GUILE_trk, (char *)SCM_ARG2,
		      NAME_midi_pitchbend);
  ch = scm_num2long (GUILE_ch, (char *)SCM_ARG3,
		     NAME_midi_pitchbend);
  val = scm_num2long (GUILE_val, (char *)SCM_ARG4,
		      NAME_midi_pitchbend);
  midi_pitchbend (m, trk, ch, val);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_set_subdiv (SCM GUILE_m, SCM GUILE_v)
{
  midi_ctx m;
  short int v;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_set_subdiv);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  v = scm_num2long (GUILE_v, (char *)SCM_ARG2,
		    NAME_midi_set_subdiv);
  midi_set_subdiv (m, v);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_meta_tempo (SCM GUILE_m, SCM GUILE_v)
{
  midi_ctx m;
  long int v;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_meta_tempo);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  v = scm_num2long (GUILE_v, (char *)SCM_ARG2,
		    NAME_midi_meta_tempo);
  midi_meta_tempo (m, v);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_meta_beat (SCM GUILE_m, SCM GUILE_n, SCM GUILE_d, SCM GUILE_c, SCM GUILE_b)
{
  midi_ctx m;
  char n, d, c, b;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_meta_beat);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  n = scm_num2long (GUILE_n, (char *)SCM_ARG2,
		    NAME_midi_meta_beat);
  d = scm_num2long (GUILE_d, (char *)SCM_ARG3,
		    NAME_midi_meta_beat);
  c = scm_num2long (GUILE_c, (char *)SCM_ARG4,
		    NAME_midi_meta_beat);
  b = scm_num2long (GUILE_b, (char *)SCM_ARG5,
		    NAME_midi_meta_beat);
  midi_meta_beat (m, n, d, c, b);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_meta_key (SCM GUILE_m, SCM GUILE_s, SCM GUILE_mj)
{
  midi_ctx m;
  char s, mj;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_meta_key);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  s = scm_num2long (GUILE_s, (char *)SCM_ARG2,
		    NAME_midi_meta_key);
  mj = scm_num2long (GUILE_mj, (char *)SCM_ARG3,
		     NAME_midi_meta_key);
  midi_meta_key (m, s, mj);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_meta_text (SCM GUILE_m, SCM GUILE_trk, SCM GUILE_s)
{
  midi_ctx m;
  int trk;
  size_t len;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_meta_text);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  trk = scm_num2long (GUILE_trk, (char *)SCM_ARG2,
	NAME_midi_meta_text);
  SCM_ASSERT (( SCM_NIMP (GUILE_s) && SCM_ROSTRINGP (GUILE_s) ),
	      GUILE_s, SCM_ARG3, NAME_midi_meta_text);
  len = SCM_LENGTH (GUILE_s);
  midi_meta_text (m, trk, len, SCM_ROCHARS (GUILE_s));
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_meta_copyright (SCM GUILE_m, SCM GUILE_s)
{
  midi_ctx m;
  size_t len;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_meta_copyright);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  SCM_ASSERT (( SCM_NIMP (GUILE_s) && SCM_ROSTRINGP (GUILE_s) ),
	GUILE_s, SCM_ARG2, NAME_midi_meta_copyright);
  len = SCM_LENGTH (GUILE_s);
  midi_meta_copyright (m, len, SCM_ROCHARS (GUILE_s));
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_meta_sequence (SCM GUILE_m, SCM GUILE_s)
{
  midi_ctx m;
  size_t len;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_meta_sequence);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  SCM_ASSERT (( SCM_NIMP (GUILE_s) && SCM_ROSTRINGP (GUILE_s) ),
	GUILE_s, SCM_ARG2, NAME_midi_meta_sequence);
  len = SCM_LENGTH (GUILE_s);
  midi_meta_sequence (m, len, SCM_ROCHARS (GUILE_s));
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_meta_trackname (SCM GUILE_m, SCM GUILE_trk, SCM GUILE_s)
{
  midi_ctx m;
  int trk;
  size_t len;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_meta_trackname);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  trk = scm_num2long (GUILE_trk, (char *)SCM_ARG2,
		      NAME_midi_meta_trackname);
  SCM_ASSERT (( SCM_NIMP (GUILE_s) && SCM_ROSTRINGP (GUILE_s) ),
	GUILE_s, SCM_ARG3, NAME_midi_meta_trackname);
  len = SCM_LENGTH (GUILE_s);
  midi_meta_trackname (m, trk, len, SCM_ROCHARS (GUILE_s));
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_meta_instrument (SCM GUILE_m, SCM GUILE_trk, SCM GUILE_s)
{
  midi_ctx m;
  int trk;
  size_t len;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_meta_instrument);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  trk = scm_num2long (GUILE_trk, (char *)SCM_ARG2,
		      NAME_midi_meta_instrument);
  SCM_ASSERT (( SCM_NIMP (GUILE_s) && SCM_ROSTRINGP (GUILE_s) ),
	GUILE_s, SCM_ARG3, NAME_midi_meta_instrument);
  len = SCM_LENGTH (GUILE_s);
  midi_meta_instrument (m, trk, len, SCM_ROCHARS (GUILE_s));
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_meta_lyrics (SCM GUILE_m, SCM GUILE_trk, SCM GUILE_s)
{
  midi_ctx m;
  int trk;
  size_t len;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_meta_lyrics);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  trk = scm_num2long (GUILE_trk, (char *)SCM_ARG2,
		      NAME_midi_meta_lyrics);
  SCM_ASSERT (( SCM_NIMP (GUILE_s) && SCM_ROSTRINGP (GUILE_s) ),
	GUILE_s, SCM_ARG3, NAME_midi_meta_lyrics);
  len = SCM_LENGTH (GUILE_s);
  midi_meta_lyrics (m, trk, len, SCM_ROCHARS (GUILE_s));
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_meta_marker (SCM GUILE_m, SCM GUILE_trk, SCM GUILE_s)
{
  midi_ctx m;
  int trk;
  size_t len;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_meta_marker);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  trk = scm_num2long (GUILE_trk, (char *)SCM_ARG2,
		      NAME_midi_meta_marker);
  SCM_ASSERT (( SCM_NIMP (GUILE_s) && SCM_ROSTRINGP (GUILE_s) ),
	GUILE_s, SCM_ARG3, NAME_midi_meta_marker);
  len = SCM_LENGTH (GUILE_s);
  midi_meta_marker (m, trk, len, SCM_ROCHARS (GUILE_s));
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_meta_cuepoint (SCM GUILE_m, SCM GUILE_trk, SCM GUILE_s)
{
  midi_ctx m;
  int trk;
  size_t len;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_meta_cuepoint);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  trk = scm_num2long (GUILE_trk, (char *)SCM_ARG2,
		      NAME_midi_meta_cuepoint);
  SCM_ASSERT (( SCM_NIMP (GUILE_s) && SCM_ROSTRINGP (GUILE_s) ),
	GUILE_s, SCM_ARG3, NAME_midi_meta_cuepoint);
  len = SCM_LENGTH (GUILE_s);
  midi_meta_cuepoint (m, trk, len, SCM_ROCHARS (GUILE_s));
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_single_note (SCM GUILE_m, SCM GUILE_trk, SCM GUILE_ch, SCM GUILE_nt, SCM GUILE_sp, SCM GUILE_d)
{
  midi_ctx m;
  int trk;
  char ch, nt, sp;
  long int d;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_single_note);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  trk = scm_num2long (GUILE_trk, (char *)SCM_ARG2,
		      NAME_midi_single_note);
  ch = scm_num2long (GUILE_ch, (char *)SCM_ARG3,
		     NAME_midi_single_note);
  nt = scm_num2long (GUILE_nt, (char *)SCM_ARG4,
		     NAME_midi_single_note);
  sp = scm_num2long (GUILE_sp, (char *)SCM_ARG5,
		     NAME_midi_single_note);
  d = scm_num2long (GUILE_d, (char *)SCM_ARG6,
		    NAME_midi_single_note);
  midi_single_note (m, trk, ch, nt, sp, d);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_start (SCM GUILE_m, SCM GUILE_tempo,
		  SCM GUILE_beat_n, SCM GUILE_beat_d, SCM GUILE_beat_c, SCM GUILE_beat_b,
		  SCM GUILE_key_s, SCM GUILE_key_m)
{
  midi_ctx m;
  long int tempo;
  char beat_n, beat_d, beat_c, beat_b, key_s, key_m;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_start);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  tempo = scm_num2long (GUILE_tempo, (char *)SCM_ARG2,
			NAME_midi_start);
  beat_n = scm_num2long (GUILE_beat_n, (char *)SCM_ARG3,
			 NAME_midi_start);
  beat_d = scm_num2long (GUILE_beat_d, (char *)SCM_ARG4,
			 NAME_midi_start);
  beat_c = scm_num2long (GUILE_beat_c, (char *)SCM_ARG5,
			 NAME_midi_start);
  beat_b = scm_num2long (GUILE_beat_b, (char *)SCM_ARG6,
			 NAME_midi_start);
  key_s = scm_num2long (GUILE_key_s, (char *)SCM_ARGn,
			NAME_midi_start);
  key_m = scm_num2long (GUILE_key_m, (char *)SCM_ARGn,
			NAME_midi_start);
  midi_start (m, tempo, beat_n, beat_d, beat_c, beat_b, key_s, key_m);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_finish (SCM GUILE_m)
{
  midi_ctx m;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_finish);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  midi_finish (m);
  return SCM_UNSPECIFIED;
}

static SCM
GUILE_midi_dump (SCM GUILE_m, SCM GUILE_port)
{
  midi_ctx m;
  FILE *f;

  SCM_ASSERT (( SCM_NIMP (GUILE_m) && SCM_CAR (GUILE_m) == SMOB_midi_tag ),
	      GUILE_m, SCM_ARG1, NAME_midi_finish);
  m = (midi_ctx) SCM_CDR (GUILE_m);
  SCM_ASSERT (( SCM_NIMP (GUILE_port) && SCM_OPFPORTP (GUILE_port) ),
	      GUILE_port, SCM_ARG2, NAME_midi_dump);
  f = (FILE *) SCM_STREAM (GUILE_port);
  midi_dump (m, f);
  return SCM_UNSPECIFIED;
}

static SCM
SMOB_mark_midi (SCM SMOB_midi)
{
  SMOB_midi = SMOB_midi;
  return SCM_BOOL_F;  /* XXX - What does that mean, exactly? */
}

static scm_sizet
SMOB_free_midi (SCM SMOB_midi)
{
  midi_ctx m;

  m = (midi_ctx)SCM_CDR (SMOB_midi);
  midi_terminate (m);
  return 0;  /* XXX - Whaaat? */
}

static int
SMOB_print_midi (SCM SMOB_midi, SCM port, scm_print_state *pstate)
{
  static char print_string[] = "#<midi>";

  SMOB_midi = SMOB_midi;
  pstate = pstate;
  scm_puts (print_string, port);
  return 1;
}

static scm_smobfuns SMOB_midi_funcs = {
  SMOB_mark_midi, SMOB_free_midi, SMOB_print_midi, 0
};

static void
init_midi_type (void)
{
  SMOB_midi_tag = scm_newsmob (&SMOB_midi_funcs);
  scm_make_gsubr (NAME_midi_init, 1, 0, 0, GUILE_midi_init);
  scm_make_gsubr (NAME_midi_wait, 3, 0, 0, GUILE_midi_wait);
  scm_make_gsubr (NAME_midi_sync, 1, 0, 0, GUILE_midi_sync);
  scm_make_gsubr (NAME_midi_all_wait, 2, 0, 0, GUILE_midi_all_wait);
  scm_make_gsubr (NAME_midi_prg, 4, 0, 0, GUILE_midi_prg);
  scm_make_gsubr (NAME_midi_ctl, 5, 0, 0, GUILE_midi_ctl);
  scm_make_gsubr (NAME_midi_n_on, 5, 0, 0, GUILE_midi_n_on);
  scm_make_gsubr (NAME_midi_n_off, 5, 0, 0, GUILE_midi_n_off);
  scm_make_gsubr (NAME_midi_pitchbend, 4, 0, 0, GUILE_midi_pitchbend);
  scm_make_gsubr (NAME_midi_set_subdiv, 2, 0, 0, GUILE_midi_set_subdiv);
  scm_make_gsubr (NAME_midi_meta_tempo, 2, 0, 0, GUILE_midi_meta_tempo);
  scm_make_gsubr (NAME_midi_meta_beat, 5, 0, 0, GUILE_midi_meta_beat);
  scm_make_gsubr (NAME_midi_meta_key, 3, 0, 0, GUILE_midi_meta_key);
  scm_make_gsubr (NAME_midi_meta_text, 3, 0, 0, GUILE_midi_meta_text);
  scm_make_gsubr (NAME_midi_meta_copyright, 2, 0, 0, GUILE_midi_meta_copyright);
  scm_make_gsubr (NAME_midi_meta_sequence, 2, 0, 0, GUILE_midi_meta_sequence);
  scm_make_gsubr (NAME_midi_meta_trackname, 3, 0, 0, GUILE_midi_meta_trackname);
  scm_make_gsubr (NAME_midi_meta_instrument, 3, 0, 0, GUILE_midi_meta_instrument);
  scm_make_gsubr (NAME_midi_meta_lyrics, 3, 0, 0, GUILE_midi_meta_lyrics);
  scm_make_gsubr (NAME_midi_meta_marker, 3, 0, 0, GUILE_midi_meta_marker);
  scm_make_gsubr (NAME_midi_meta_cuepoint, 3, 0, 0, GUILE_midi_meta_cuepoint);
  scm_make_gsubr (NAME_midi_single_note, 6, 0, 0, GUILE_midi_single_note);
  scm_make_gsubr (NAME_midi_start, 8, 0, 0, GUILE_midi_start);
  scm_make_gsubr (NAME_midi_finish, 1, 0, 0, GUILE_midi_finish);
  scm_make_gsubr (NAME_midi_dump, 2, 0, 0, GUILE_midi_dump);
}

static void
inner_main (void *closure, int argc, char *argv[])
{
  closure = NULL;
  init_midi_type ();
  scm_shell (argc, argv);
}

int
main (int argc, char *argv[])
{
  scm_boot_guile (argc, argv, inner_main, 0);
  return 0;  /* Unreachable */
}
