;;; ******************************************************************
;;; *                                                                *
;;; *               Johann Sebastian Bach (1685-1750)                *
;;; *                   Prelude in C major, BWV846                   *
;;; *                (From the Well-Tempered Clavier)                *
;;; *             Sequenced by David Madore, 2000/07/21              *
;;; *                    ---  Public Domain  ---                     *
;;; *                                                                *
;;; ******************************************************************

(define m (midi-init 2))
;; Track 1 is right hand, track 2 is left hand.

(midi-set-subdiv m 120)
(define base-tempo 700000)
(midi-start m base-tempo  4 2 120 8  0 0)  ;; C major

(midi-meta-sequence m "Johann Sebastian Bach (1685-1750)")
(midi-meta-sequence m "Prelude in C major, BWV846")
(midi-meta-sequence m "(From the Well-Tempered Clavier)")
(midi-meta-text m 0 "Sequenced by David Madore, 2000/07/21")
(midi-meta-copyright m "Public Domain")

(midi-meta-trackname m 1 "Right hand")
(midi-meta-trackname m 2 "Left hand")

(define interp 'piano)

(case interp
  ((harpsichord)
   (midi-meta-instrument m 1 "Harpsichord")
   (midi-meta-instrument m 2 "Harpsichord")
   (midi-prg m 1 0 6)
   (midi-ctl m 1 0 7 100)
   (midi-ctl m 1 0 10 68)
   (midi-prg m 2 1 6)
   (midi-ctl m 2 1 7 100)
   (midi-ctl m 2 1 10 60)
   ;; Use the true baroque pitch of 415Hz (rather than 440Hz): this is
   ;; 4148/4096 halftones below the usual pitch.
   (midi-pitchbend m 1 0 4148)
   (midi-pitchbend m 2 1 4148))
  ((piano)
   (midi-meta-instrument m 1 "Piano")
   (midi-meta-instrument m 2 "Piano")
   (midi-prg m 1 0 0)
   (midi-ctl m 1 0 7 100)
   (midi-ctl m 1 0 10 68)
   (midi-prg m 2 1 0)
   (midi-ctl m 2 1 7 100)
   (midi-ctl m 2 1 10 60))
  ((organ)
   (midi-meta-instrument m 1 "Organ")
   (midi-meta-instrument m 2 "Organ")
   (midi-prg m 1 0 19)
   (midi-ctl m 1 0 7 100)
   (midi-ctl m 1 0 10 72)
   (midi-prg m 2 1 19)
   (midi-ctl m 2 1 7 100)
   (midi-ctl m 2 1 10 56))
  ((strings)
   (midi-meta-instrument m 1 "Violin")
   (midi-meta-instrument m 2 "Cello")
   (midi-prg m 1 0 40)
   (midi-ctl m 1 0 7 120)
   (midi-ctl m 1 0 10 72)
   (midi-prg m 2 1 42)
   (midi-ctl m 2 1 7 60)
   (midi-ctl m 2 1 10 56))
  ((brass)
   (midi-meta-instrument m 1 "Trumpet")
   (midi-meta-instrument m 2 "Tuba")
   (midi-prg m 1 0 56)
   (midi-ctl m 1 0 7 90)
   (midi-ctl m 1 0 10 72)
   (midi-prg m 2 1 58)
   (midi-ctl m 2 1 7 100)
   (midi-ctl m 2 1 10 56)))

;; Chorus and reverb.
(midi-ctl m 1 0 91 50)
(midi-ctl m 1 0 93 50)
(midi-ctl m 2 1 91 50)
(midi-ctl m 2 1 93 70)

(define (midi-multiple-notes m trk ch nt-tab sp d)
  (let loop ((t nt-tab))
    (if (pair? t)
        (let ((nt (car t)))
          (if nt
              (midi-single-note m trk ch nt sp d)
              (midi-wait m trk d))
          (loop (cdr t))))))

(define (midi-symbolic n)
  (let loop ((nt 0)
             (oct 5)
             (alt 0)
             (pl #f)
             (t n))
    (if (pair? t)
        (let ((e (car t)))
          (case e
            ((#f) (loop nt oct alt #f (cdr t)))
            ((c) (loop 0 oct alt #t (cdr t)))
            ((d) (loop 2 oct alt #t (cdr t)))
            ((e) (loop 4 oct alt #t (cdr t)))
            ((f) (loop 5 oct alt #t (cdr t)))
            ((g) (loop 7 oct alt #t (cdr t)))
            ((a) (loop 9 oct alt #t (cdr t)))
            ((b) (loop 11 oct alt #t (cdr t)))
            ((na) (loop nt oct 0 pl (cdr t)))
            ((sh) (loop nt oct (+ alt 1) pl (cdr t)))
            ((fl) (loop nt oct (- alt 1) pl (cdr t)))
            ((sh2) (loop nt oct (+ alt 2) pl (cdr t)))
            ((fl2) (loop nt oct (- alt 2) pl (cdr t)))
            ((0 1 2 3 4 5 6 7 8 9) (loop nt e alt pl (cdr t)))
            (else 'unknown)))
        (and pl (+ nt alt (* oct 12))))))

(define (prelC-bar nt1 nt2 nt3 nt4 nt5)
  (define (half)
    (midi-n-on m 2 1 nt1 100)
    (midi-wait m 2 30)
    (midi-single-note m 2 1 nt2 100 210)
    (midi-n-off m 2 1 nt1 0)
    (midi-multiple-notes m 1 0 (list #f #f nt3 nt4 nt5 nt3 nt4 nt5) 100 30))
  (half) (half))

(apply prelC-bar (map midi-symbolic '((c 5) (e 5) (g 5) (c 6) (e 6)))) ;; 1
(apply prelC-bar (map midi-symbolic '((c 5) (d 5) (a 5) (d 6) (f 6)))) ;; 2
(apply prelC-bar (map midi-symbolic '((b 4) (d 5) (g 5) (d 6) (f 6)))) ;; 3
(apply prelC-bar (map midi-symbolic '((c 5) (e 5) (g 5) (c 6) (e 6)))) ;; 4
(apply prelC-bar (map midi-symbolic '((c 5) (e 5) (a 5) (e 6) (a 6)))) ;; 5
(apply prelC-bar (map midi-symbolic '((c 5) (d 5) (f sh 5) (a 5) (d 6)))) ;; 6
(apply prelC-bar (map midi-symbolic '((b 4) (d 5) (g 5) (d 6) (g 6)))) ;; 7
(apply prelC-bar (map midi-symbolic '((b 4) (c 5) (e 5) (g 5) (c 6)))) ;; 8
(apply prelC-bar (map midi-symbolic '((a 4) (c 5) (e 5) (g 5) (c 6)))) ;; 9
(apply prelC-bar (map midi-symbolic '((d 4) (a 4) (d 5) (f sh 5) (c 6)))) ;; 10
(apply prelC-bar (map midi-symbolic '((g 4) (b 4) (d 5) (g 5) (b 5)))) ;; 11
(apply prelC-bar (map midi-symbolic '((g 4) (b fl 4) (e 5) (g 5) (c sh 6)))) ;; 12
(apply prelC-bar (map midi-symbolic '((f 4) (a 4) (d 5) (a 5) (d 6)))) ;; 13
(apply prelC-bar (map midi-symbolic '((f 4) (a fl 4) (d 5) (f 5) (b 5)))) ;; 14
(apply prelC-bar (map midi-symbolic '((e 4) (g 4) (c 5) (g 5) (c 6)))) ;; 15
(apply prelC-bar (map midi-symbolic '((e 4) (f 4) (a 4) (c 5) (f 5)))) ;; 16
(apply prelC-bar (map midi-symbolic '((d 4) (f 4) (a 4) (c 5) (f 5)))) ;; 17
(apply prelC-bar (map midi-symbolic '((g 3) (d 4) (g 4) (b 4) (f 5)))) ;; 18
(apply prelC-bar (map midi-symbolic '((c 4) (e 4) (g 4) (c 5) (e 5)))) ;; 19
(apply prelC-bar (map midi-symbolic '((c 4) (g 4) (b fl 4) (c 5) (e 5)))) ;; 20
(apply prelC-bar (map midi-symbolic '((f 3) (f 4) (a 4) (c 5) (e 5)))) ;; 21
(apply prelC-bar (map midi-symbolic '((f sh 3) (c 4) (a 4) (c 5) (e fl 5)))) ;; 22
(apply prelC-bar (map midi-symbolic '((a fl 3) (f 4) (b 4) (c 5) (d 5)))) ;; 23
(apply prelC-bar (map midi-symbolic '((g 3) (f 4) (g 4) (b 4) (d 5)))) ;; 24
(apply prelC-bar (map midi-symbolic '((g 3) (e 4) (g 4) (c 5) (e 5)))) ;; 25
(apply prelC-bar (map midi-symbolic '((g 3) (d 4) (g 4) (c 5) (f 5)))) ;; 26
(apply prelC-bar (map midi-symbolic '((g 3) (d 4) (g 4) (b 4) (f 5)))) ;; 27
(apply prelC-bar (map midi-symbolic '((g 3) (e fl 4) (a 4) (c 5) (f sh 5)))) ;; 28
(apply prelC-bar (map midi-symbolic '((g 3) (e 4) (g 4) (c 5) (g 5)))) ;; 29
(apply prelC-bar (map midi-symbolic '((g 3) (d 4) (g 4) (c 5) (f 5)))) ;; 30
(apply prelC-bar (map midi-symbolic '((g 3) (d 4) (g 4) (b 4) (f 5)))) ;; 31
(apply prelC-bar (map midi-symbolic '((c 3) (c 4) (g 4) (b fl 4) (e 5)))) ;; 32

;; 33
(midi-n-on m 2 1 (midi-symbolic '(c 3)) 100)
(midi-wait m 2 30)
(midi-single-note m 2 1 (midi-symbolic '(c 4)) 100 450)
(midi-n-off m 2 1 (midi-symbolic '(c 3)) 0)
(midi-multiple-notes m 1 0 (map midi-symbolic '(
  ()    ()    (f 4) (a 4) (c 5) (f 5) (c 5) (a 4)
  (c 5) (a 4) (f 4) (a 4) (f 4) (d 4) (f 4) (d 4)
)) 100 30)

;; 34
(midi-n-on m 2 1 (midi-symbolic '(c 3)) 100)
(midi-wait m 2 30)
(midi-single-note m 2 1 (midi-symbolic '(b 3)) 100 450)
(midi-n-off m 2 1 (midi-symbolic '(c 3)) 0)
(midi-multiple-notes m 1 0 (map midi-symbolic '(
  ()    ()    (g 5) (b 5) (d 6) (f 6) (d 6) (b 5)
  (d 6) (b 5) (g 5) (b 5) (d 5) (f 5) (e 5) (d 5)
)) 100 30)

;; 35
(midi-n-on m 2 1 (midi-symbolic '(c 3)) 100)
(midi-n-on m 2 1 (midi-symbolic '(c 4)) 100)
(midi-wait m 2 480)
(midi-n-on m 2 1 (midi-symbolic '(c 3)) 0)
(midi-n-on m 2 1 (midi-symbolic '(c 4)) 0)

(midi-n-on m 1 0 (midi-symbolic '(e 5)) 100)
(midi-n-on m 1 0 (midi-symbolic '(g 5)) 100)
(midi-n-on m 1 0 (midi-symbolic '(c 6)) 100)
(midi-wait m 1 480)
(midi-n-on m 1 0 (midi-symbolic '(e 5)) 0)
(midi-n-on m 1 0 (midi-symbolic '(g 5)) 0)
(midi-n-on m 1 0 (midi-symbolic '(c 6)) 0)

;(midi-sync m)  ;; Should be no-op

(midi-finish m)
(midi-dump m (current-output-port))
