;;; ******************************************************************
;;; *                                                                *
;;; *               Johann Sebastian Bach (1685-1750)                *
;;; *                   Prelude in D major, BWV850                   *
;;; *                (From the Well-Tempered Clavier)                *
;;; *             Sequenced by David Madore, 2000/07/21              *
;;; *                    ---  Public Domain  ---                     *
;;; *                                                                *
;;; ******************************************************************

(define m (midi-init 2))
;; Track 1 is right hand, track 2 is left hand.

(midi-set-subdiv m 120)
(define base-tempo 540975)
(midi-start m base-tempo  4 2 120 8  2 0)  ;; D major

(midi-meta-sequence m "Johann Sebastian Bach (1685-1750)")
(midi-meta-sequence m "Prelude in D major, BWV850")
(midi-meta-sequence m "(From the Well-Tempered Clavier)")
(midi-meta-text m 0 "Sequenced by David Madore, 2000/07/21")
(midi-meta-copyright m "Public Domain")

(midi-meta-trackname m 1 "Right hand")
(midi-meta-trackname m 2 "Left hand")

(define interp 'organ)

(case interp
  ((harpsichord)
   (midi-meta-instrument m 1 "Harpsichord")
   (midi-meta-instrument m 2 "Harpsichord")
   (midi-prg m 1 0 6)
   (midi-ctl m 1 0 7 100)
   (midi-ctl m 1 0 10 68)
   (midi-prg m 2 1 6)
   (midi-ctl m 2 1 7 100)
   (midi-ctl m 2 1 10 60)
   ;; Use the true baroque pitch of 415Hz (rather than 440Hz): this is
   ;; 4148/4096 halftones below the usual pitch.
   (midi-pitchbend m 1 0 4148)
   (midi-pitchbend m 2 1 4148))
  ((piano)
   (midi-meta-instrument m 1 "Piano")
   (midi-meta-instrument m 2 "Piano")
   (midi-prg m 1 0 0)
   (midi-ctl m 1 0 7 100)
   (midi-ctl m 1 0 10 68)
   (midi-prg m 2 1 0)
   (midi-ctl m 2 1 7 100)
   (midi-ctl m 2 1 10 60))
  ((organ)
   (midi-meta-instrument m 1 "Organ")
   (midi-meta-instrument m 2 "Organ")
   (midi-prg m 1 0 19)
   (midi-ctl m 1 0 7 100)
   (midi-ctl m 1 0 10 72)
   (midi-prg m 2 1 19)
   (midi-ctl m 2 1 7 100)
   (midi-ctl m 2 1 10 56))
  ((strings)
   (midi-meta-instrument m 1 "Violin")
   (midi-meta-instrument m 2 "Cello")
   (midi-prg m 1 0 40)
   (midi-ctl m 1 0 7 120)
   (midi-ctl m 1 0 10 72)
   (midi-prg m 2 1 42)
   (midi-ctl m 2 1 7 60)
   (midi-ctl m 2 1 10 56))
  ((brass)
   (midi-meta-instrument m 1 "Trumpet")
   (midi-meta-instrument m 2 "Tuba")
   (midi-prg m 1 0 56)
   (midi-ctl m 1 0 7 90)
   (midi-ctl m 1 0 10 72)
   (midi-prg m 2 1 58)
   (midi-ctl m 2 1 7 100)
   (midi-ctl m 2 1 10 56)))

;; Chorus and reverb.
(midi-ctl m 1 0 91 50)
(midi-ctl m 1 0 93 50)
(midi-ctl m 2 1 91 50)
(midi-ctl m 2 1 93 70)

(define (midi-multiple-notes m trk ch nt-tab sp d)
  (let loop ((t nt-tab))
    (if (pair? t)
        (let ((nt (car t)))
          (if nt
              (midi-single-note m trk ch nt sp d)
              (midi-wait m trk d))
          (loop (cdr t))))))

(define (midi-symbolic n)
  (let loop ((nt 0)
             (oct 5)
             (alt 0)
             (pl #f)
             (t n))
    (if (pair? t)
        (let ((e (car t)))
          (case e
            ((#f) (loop nt oct alt #f (cdr t)))
            ((c) (loop 0 oct alt #t (cdr t)))
            ((d) (loop 2 oct alt #t (cdr t)))
            ((e) (loop 4 oct alt #t (cdr t)))
            ((f) (loop 5 oct alt #t (cdr t)))
            ((g) (loop 7 oct alt #t (cdr t)))
            ((a) (loop 9 oct alt #t (cdr t)))
            ((b) (loop 11 oct alt #t (cdr t)))
            ((na) (loop nt oct 0 pl (cdr t)))
            ((sh) (loop nt oct (+ alt 1) pl (cdr t)))
            ((fl) (loop nt oct (- alt 1) pl (cdr t)))
            ((sh2) (loop nt oct (+ alt 2) pl (cdr t)))
            ((fl2) (loop nt oct (- alt 2) pl (cdr t)))
            ((0 1 2 3 4 5 6 7 8 9) (loop nt e alt pl (cdr t)))
            (else 'unknown)))
        (and pl (+ nt alt (* oct 12))))))

(midi-multiple-notes m 1 0 (map midi-symbolic '(
;; Bar 1
  ()     (d 6)  (e 6)  (f sh 6)  (a 5)  (f sh 6)  (e 6)  (d 6)
  (a 6)  (d 6)  (e 6)  (f sh 6)  (a 5)  (f sh 6)  (e 6)  (d 6)
;; Bar 2
  (g 6)     (e 6)  (f sh 6)  (g 6)  (b 5)  (d 6)  (c sh 6)  (b 5)
  (c sh 6)  (e 6)  (f sh 6)  (g 6)  (a 5)  (g 6)  (f sh 6)  (e 6)
;; Bar 3
  (f sh 6)  (d 6)  (e 6)  (f sh 6)  (a 5)  (c sh 6)  (b 5)  (a 5)
  (b 5)     (d 6)  (e 6)  (f sh 6)  (b 5)  (a 6)  (g sh 6)  (f sh 6)
;; Bar 4
  (g sh 6)  (e 6)  (f sh 6)  (g sh 6)  (b 5)  (d 6)  (c sh 6)  (b 5)
  (a 5)     (c sh 6)  (d 6)  (e 6)     (f sh 5)  (e 6)  (d 6)  (c sh 6)
;; Bar 5
  (d 6)     (b 5)  (c sh 6)  (d 6)  (f sh 5)  (a 5)  (g sh 5)  (f sh 5)
  (g sh 5)  (b 5)  (c sh 6)  (d 6)  (e 5)     (d 6)  (c sh 6)  (b 5)
;; Bar 6
  (c sh 6)  (a 5)  (b 5)  (c sh 6)  (e 5)     (g 5)  (f sh 5)  (e 5)
  (f sh 5)  (a 5)  (b 5)  (c sh 6)  (f sh 5)  (e 6)  (d sh 6)  (c sh 6)
)) 100 30)

(midi-multiple-notes m 2 1 (map midi-symbolic '(
;; Bar 1
  (d 4)         ()               (d 5)            ()
  (f sh 4)      ()               (d 5)            ()
;; Bar 2
  (e 4)            ()               (d 5)         ()
  (a 4)            ()               (c sh 5)      ()
;; Bar 3
  (d 4)            ()               (d 5)         ()
  (d 4)            ()               (d 5)         ()
;; Bar 4
  (d 4)            ()                  (g sh 4)      ()
  (c sh 4)         ()                  (a 4)         ()
;; Bar 5
  (b 3)            ()               (a 4)            ()
  (e 4)            ()               (g sh 4)         ()
;; Bar 6
  (a 3)            ()               (a 4)            ()
  (a 3)            ()               (a 4)            ()
)) 100 60)

;(midi-sync m)  ;; Should be no-op

(midi-multiple-notes m 1 0 (map midi-symbolic '(
;; Bar 7
  (d sh 6)  (b 5)  (c sh 6)  (d sh 6)  (f sh 5)  (a 5)  (g 5)  (f sh 5)
  (e 5)     (g 5)  (a 5)     (b 5)     (c sh 5)  (d 6)  (c sh 6)  (b 5)
;; Bar 8
  (a sh 5)  (e 5)  (f sh 5)  (g 5)  (c sh 5)  (g 5)  (f sh 5)  (e 5)
  (d 5)     (b 5)  (c sh 6)  (d 6)  (f sh 5)  (a 5)  (g sh 5)  (f sh 5)
;; Bar 9
  (e sh 5)  (g sh 5)  (a 5)     (b 5)     (e sh 5)  (d 6)  (c sh 6)  (b 5)
  (g sh 6)  (e sh 6)  (f sh 6)  (g sh 6)  (c sh 6)  (b 6)  (a 6)  (g sh 6)
;; Bar 10
  (a 6)  (f sh 6)  (g sh 6)  (a 6)  (c sh 6)  (e 6)  (d sh 6)  (c sh 6)
  (d sh 6)  (f sh 6)  (g 6)  (a 6)  (b 5)     (a 6)  (g 6)  (f sh 6)
;; Bar 11
  (g 6)     (e 6)  (f sh 6)  (g 6)     (b 5)  (d 6)  (c na 6)  (b 5)
  (c na 6)  (d sh 6)  (e 6)  (f sh 6)  (a 5)  (c na 6)  (b 5)  (a 5)
;; Bar 12
  (g 5)     (e 6)  (f sh 6)  (g 6)  (b 5)  (d 6)  (c sh 6)  (b 5)
  (c sh 6)  (e 6)  (f sh 6)  (g 6)  (a 5)  (g 6)  (f sh 6)  (e 6)
)) 100 30)

(midi-multiple-notes m 2 1 (map midi-symbolic '(
;; Bar 7
  (a 3)            ()                  (d sh 4)         ()
  (g 3)            ()                  (e 4)            ()
;; Bar 8
  (f sh 3)         ()               (f sh 4)         ()
  (f sh 3)         ()               (f sh 4)         ()
;; Bar 9
  (f sh 3)         ()                     (f sh 4)         ()
  (f sh 3)         ()                     (f sh 4)         ()
;; Bar 10
  (f sh 3)         ()               (f sh 4)         ()
  (b 3)            ()               (d sh 4)         ()
;; Bar 11
  (e 3)            ()                  (e 4)         ()
  (e 3)            ()                  (d sh 4)         ()
;; Bar 12
  (e 3)            ()               (e 4)         ()
  (a 3)            ()               (c sh 4)      ()
)) 100 60)

;(midi-sync m)  ;; Should be no-op

(midi-multiple-notes m 1 0 (map midi-symbolic '(
;; Bar 13
  (f sh 6)  (d 6)  (e 6)  (f sh 6)  (a 5)  (c na 6)  (b 5)  (a 5)
  (b 5)     (g 5)  (a 5)  (b 5)     (c sh 5)  (b 5)  (a 5)  (g 5)
;; Bar 14
  (f sh 5)  (d 5)  (e 5)  (f sh 5)  (a 4)  (c na 5)  (b 4)  (a 4)
  (b 4)     (d 5)  (e 5)  (f sh 5)  (b 4)  (a 5)  (g sh 5)  (f sh 5)
;; Bar 15
  (g sh 5)  (e 5)  (f sh 5)  (g sh 5)  (b 4)  (d 5)  (c na 5)  (b 4)
  (a 4)  (c na 5)  (d 5)     (e 5)     (a 4)  (g 5)  (f sh 5)  (e 5)
;; Bar 16
  (d sh 5)  (a 4)  (b 4)  (c na 5)  (f sh 4)  (c na 5)  (b 4)  (a 4)
  (g 4)  (e 5)  (f sh 5)  (g 5)     (b 4)  (d 5)  (c sh 5)  (b 4)
;; Bar 17
  (a sh 4)  (c sh 5)  (d 5)  (e 5)     (a sh 4)  (g 5)  (f sh 5)  (e 5)
  (c sh 6)  (a sh 5)  (b 5)  (c sh 6)  (f sh 5)  (e 6)  (d 6)  (c sh 6)
;; Bar 18
  (d 6)     (b 5)  (c sh 6)  (d 6)  (f sh 5)  (a 5)  (g sh 5)  (f sh 5)
  (g sh 5)  (b 5)  (c na 6)  (d 6)  (e 5)     (d 6)  (c na 6)  (b 5)
)) 100 30)

(midi-multiple-notes m 2 1 (map midi-symbolic '(
;; Bar 13
  (d 3)            ()               (d 4)            ()
  (d 3)            ()               (d 4)            ()
;; Bar 14
  (d 3)            ()               (d 4)            ()
  (d 3)            ()               (d 4)            ()
;; Bar 15
  (d 3)            ()                  (g sh 4)         ()
  (c na 4)         ()                  (f sh 4)         ()
;; Bar 16
  (b 3)               ()               (d sh 4)         ()
  (e 4)               ()               (g 4)            ()
;; Bar 17
  (b 3)               ()               (e 4)            ()
  (g 4)               ()               (a sh 4)         ()
;; Bar 18
  (b 3)            ()               (b 4)            ()
  (e 4)            ()               (g sh 4)         ()
)) 100 60)

;(midi-sync m)  ;; Should be no-op

(midi-multiple-notes m 1 0 (map midi-symbolic '(
;; Bar 19
  (c na 6)  (a 5)  (b 5)  (c na 6)  (e 5)  (g 5)     (f sh 5)  (e 5)
  (f sh 5)  (a 5)  (b 5)  (c na 6)  (d 5)  (c na 6)  (b 5)  (a 5)
;; Bar 20
  (b 5)  (g 5)  (a 5)  (b 5)     (d 5)  (b 5)  (a 5)  (g 5)
  (d 6)  (g 5)  (a 5)  (b 5)     (d 5)  (b 5)  (a 5)  (g 5)
;; Bar 21
  (c na 6)  (a 5)  (b 5)  (c na 6)  (e 5)  (g 5)     (f sh 5)  (e 5)
  (f sh 5)  (a 5)  (b 5)  (c na 6)  (d 5)  (c na 6)  (b 5)  (a 5)
;; Bar 22
  (b 5)  (g 5)  (a 5)  (b 5)     (d 5)  (f sh 5)  (e 5)  (d 5)
  (e 5)  (g 5)  (a 5)  (b 5)     (e 5)  (d 6)  (c sh 6)  (b 5)
;; Bar 23
  (c sh 6)  (a 5)  (b 5)  (c sh 6)  (e 5)  (g 5)  (f sh 5)  (e 5)
  (d 5)  (f sh 5)  (g 5)  (a 5)     (b 4)  (a 5)  (g 5)  (f sh 5)
;; Bar 24
  (g 5)     (e 5)  (f sh 5)  (g 5)  (b 4)  (d 5)  (c sh 5)  (b 4)
  (c sh 5)  (e 5)  (f sh 5)  (g 5)  (a 4)  (g 5)  (f sh 5)  (e 5)
;; Bar 25
  (f sh 5)  (d 5)  (e 5)  (f sh 5)  (a 4)  (f sh 5)  (e 5)  (d 5)
  (a 5)  (f sh 5)  (g 5)  (a 5)     (d 5)  (c na 6)  (b 5)  (a 5)
;; Bar 26
  (b 5)  (g 5)  (a 5)  (b 5)     (d 5)     (b 5)     (a 5)  (g 5)
  (d 6)  (b 5)  (c sh 6)  (d 6)  (g sh 5)  (f na 6)  (e 6)  (d 6)
)) 100 30)

(midi-multiple-notes m 2 1 (map midi-symbolic '(
;; Bar 19
  (a 3)            ()               (a 4)            ()
  (d 4)            ()               (f sh 4)         ()
;; Bar 20
  (g 3)            ()               (g 4)            ()
  (b 3)            ()               (g 4)            ()
;; Bar 21
  (a 3)            ()               (g 4)            ()
  (d 4)            ()               (f sh 4)         ()
;; Bar 22
  (g 3)         ()               (g 4)            ()
  (g 3)         ()               (g 4)            ()
;; Bar 23
  (g 3)            ()               (c sh 4)      ()
  (f sh 3)         ()               (d 4)         ()
;; Bar 24
  (e 3)            ()               (d 4)         ()
  (a 3)            ()               (c sh 4)      ()
;; Bar 25
  (d 3)            ()               (d 4)            ()
  (f sh 3)         ()               (a 4)            ()
;; Bar 26
  (g 3)         ()               (g 4)               ()
  (b 3)         ()               (b 4)               ()
)) 100 60)

(midi-sync m)  ;; (Bring track 0 to date for tempo event)

(midi-meta-tempo m (quotient (* base-tempo 21) 20))

(midi-multiple-notes m 1 0 (map midi-symbolic '(
;; Bar 27
  (c sh 6)  (a 5)  (b 5)  (c sh 6)  (e 5)  (g 5)  (f sh 5)  (e 5)
  (a 5)     (d 6)  (e 6)  (f sh 6)  (b 5)  (a 6)  (g 6)  (f sh 6)
;; Bar 28
  (g 6)  (c sh 6)  (d 6)  (e 6)     (a 5)  (g 6)  (f sh 6)  (e 6)
  (f sh 6)  (b 5)  (c sh 6)  (d 6)  (g sh 5)  (f sh 6)  (e 6)  (d 6)
;; Bar 29
  (c sh 6)  (g 5)  (a 5)  (b fl 5)  (e 5)  (g 5)  (f na 5)  (e 5)
  (f na 5)  (d 6)  (e 6)  (f na 6)  (c sh 6)  (e 6)  (d 6)  (c sh 6)
;; Bar 30
  (d 6)  (b fl 6)  (a 6)  (g 6)  (f na 6)  (e 6)  (d 6)  (c sh 6)
  (d 6)  (f sh 6)  (e 6)  (d 6)  (b 6)  (g sh 6)  (a 6)  (b 6)
;; Bar 31
  (f na 6)  (d 6)  (e 6)  (f na 6)  (b 5)  (g sh 5)  (a 5)  (b 5)
  (e 5)  (f sh 5)  (g sh 5)  (a 5)  (b 5)  (c sh 6)  (d 6)  (e 6)
)) 100 30)

(midi-n-on m 2 1 (midi-symbolic '(a 3)) 80)
(midi-multiple-notes m 2 1 (map midi-symbolic '(
;; Bar 27
  (a 3)            ()               (g 4)            ()
  (f sh 4)         ()               (d 4)            ()
;; Bar 28
  (e 4)            ()               (c sh 4)         ()
  (d 4)            ()               (b 3)            ()
)) 100 60)
(midi-multiple-notes m 2 1 (map midi-symbolic '(
;; Bar 29
  ()  (e 4)     (f sh 4)  (g 4)  (c sh 4)  (e 4)  (d 4)  (c sh 4)
  (d 4)  (f na 4)  (g 4)  (a 4)  (e 4)  (g 4)  (f na 4)  (e 4)
)) 100 30)
(midi-n-on m 2 1 (midi-symbolic '(a 3)) 0)
(midi-multiple-notes m 2 1 (map midi-symbolic '(
;; Bar 30
  (f na 4)  (d 4)  (e 4)  (f na 4)  (a 3)  (c na 4)  (b fl 3)  (a 3)
)) 100 30)
(midi-single-note m 2 1 (midi-symbolic '(g sh 3)) 100 720)

(midi-sync m)  ;; (Bring track 0 to date for tempo event)

(midi-meta-tempo m (quotient (* base-tempo 22) 20))

(midi-n-on m 1 0 (midi-symbolic '(f na 6)) 80)
(midi-multiple-notes m 1 0 (map midi-symbolic '(
;; Bar 32
  (f na 6)  (c sh 6)  (d 6)  (a 5)  (b fl 5)  (g 5)
)) 100 30)
(midi-n-on m 1 0 (midi-symbolic '(f na 6)) 0)
(midi-n-on m 1 0 (midi-symbolic '(e 6)) 80)
(midi-multiple-notes m 1 0 (map midi-symbolic '(
;; Bar 32 <contd>
                                                     (e 6)  (g 5)
  (a 5)  (f na 5)
)) 100 30)
(midi-n-on m 1 0 (midi-symbolic '(e 6)) 0)
(midi-n-on m 1 0 (midi-symbolic '(d 6)) 80)
(midi-multiple-notes m 1 0 (map midi-symbolic '(
;; Bar 32 <contd>
                   (d 6)  (f na 5)  (g 5)  (e 5)
)) 100 30)
(midi-n-on m 1 0 (midi-symbolic '(d 6)) 0)
(midi-n-on m 1 0 (midi-symbolic '(c sh 6)) 80)
(midi-multiple-notes m 1 0 (map midi-symbolic '(
;; Bar 32 <contd>
                                                  (c sh 6)  (e 5)
)) 100 30)
(midi-n-on m 1 0 (midi-symbolic '(c sh 6)) 0)

(midi-n-on m 2 1 (midi-symbolic '(a 3)) 80)
(midi-multiple-notes m 2 1 (map midi-symbolic '(
;; Bar 32
  ()                  (a 4)         (g 4)            (c sh 4)
  (f na 4)         (d 4)            (e 4)         (g 4)
)) 100 60)
(midi-n-on m 2 1 (midi-symbolic '(a 3)) 0)

(midi-sync m)  ;; (Bring track 0 to date for tempo event)

;; --- Bar 33 ---

(midi-meta-tempo m (quotient (* base-tempo 24) 20))

(midi-n-on m 2 1 (midi-symbolic '(a 3)) 80) (midi-wait m 2 5)
(midi-n-on m 2 1 (midi-symbolic '(d 4)) 80) (midi-wait m 2 3)
(midi-n-on m 2 1 (midi-symbolic '(f na 4)) 80) (midi-wait m 2 3)
(midi-n-on m 2 1 (midi-symbolic '(g 4)) 80) (midi-wait m 2 3)
(midi-n-on m 2 1 (midi-symbolic '(b 4)) 80)
(midi-wait m 2 106)
(midi-n-on m 2 1 (midi-symbolic '(a 3)) 0)
(midi-n-on m 2 1 (midi-symbolic '(d 4)) 0)
(midi-n-on m 2 1 (midi-symbolic '(f na 4)) 0)
(midi-n-on m 2 1 (midi-symbolic '(g 4)) 0)
(midi-n-on m 2 1 (midi-symbolic '(b 4)) 0)
(midi-wait m 2 360)

(midi-wait m 1 17) ;; Wait for left hand
(midi-n-on m 1 0 (midi-symbolic '(d 5)) 80) (midi-wait m 1 5)
(midi-n-on m 1 0 (midi-symbolic '(f na 5)) 80) (midi-wait m 1 3)
(midi-n-on m 1 0 (midi-symbolic '(g sh 5)) 80) (midi-wait m 1 3)
(midi-n-on m 1 0 (midi-symbolic '(b 5)) 80) (midi-wait m 1 3)
(midi-n-on m 1 0 (midi-symbolic '(d 6)) 80)
(midi-wait m 1 89)
(midi-n-on m 1 0 (midi-symbolic '(d 5)) 0)
(midi-n-on m 1 0 (midi-symbolic '(f na 5)) 0)
(midi-n-on m 1 0 (midi-symbolic '(g sh 5)) 0)
(midi-n-on m 1 0 (midi-symbolic '(b 5)) 0)
(midi-wait m 1 15)
(midi-n-on m 1 0 (midi-symbolic '(d 6)) 0)
(midi-multiple-notes m 1 0 (map midi-symbolic '(
  (c sh 6) (b 5) (a 5)  (g sh 5) (f sh 5) (e 5) (f sh 5)
  (g sh 5) (a 5) (b 5) (c sh 6)  (d 6) (e 6) (f na 6) (e 6)
  (d 6) (c sh 6) (b 5) (a 5)  (g sh 5) (f sh 5) (e 5) (d 5)
)) 100 15)

(midi-sync m)  ;; (Bring track 0 to date for tempo event)

;; --- Bar 34 ---

(midi-meta-tempo m (quotient (* base-tempo 28) 20))

(midi-n-on m 2 1 (midi-symbolic '(c sh 5)) 80) (midi-wait m 2 5)
(midi-n-on m 2 1 (midi-symbolic '(e 5)) 80) (midi-wait m 2 5)
(midi-n-on m 2 1 (midi-symbolic '(g 5)) 80) (midi-wait m 2 5)
(midi-n-on m 2 1 (midi-symbolic '(b fl 5)) 80)
(midi-wait m 2 45)
(midi-n-on m 2 1 (midi-symbolic '(c sh 5)) 0)
(midi-n-on m 2 1 (midi-symbolic '(e 5)) 0)
(midi-n-on m 2 1 (midi-symbolic '(g 5)) 0)
(midi-n-on m 2 1 (midi-symbolic '(b fl 5)) 0)
(midi-wait m 2 60)
(midi-n-on m 2 1 (midi-symbolic '(d 5)) 80) (midi-wait m 2 5)
(midi-n-on m 2 1 (midi-symbolic '(f na 5)) 80) (midi-wait m 2 5)
(midi-n-on m 2 1 (midi-symbolic '(g sh 5)) 80) (midi-wait m 2 5)
(midi-n-on m 2 1 (midi-symbolic '(b 5)) 80)
(midi-wait m 2 45)
(midi-n-on m 2 1 (midi-symbolic '(d 5)) 0)
(midi-n-on m 2 1 (midi-symbolic '(f na 5)) 0)
(midi-n-on m 2 1 (midi-symbolic '(g sh 5)) 0)
(midi-n-on m 2 1 (midi-symbolic '(b 5)) 0)
(midi-wait m 2 60)

(midi-n-on m 2 1 (midi-symbolic '(a 3)) 100)
(midi-n-on m 2 1 (midi-symbolic '(e 4)) 100)
(midi-n-on m 2 1 (midi-symbolic '(a 4)) 100)
(midi-wait m 2 120)
(midi-n-on m 2 1 (midi-symbolic '(e 4)) 0)
(midi-n-on m 2 1 (midi-symbolic '(g 4)) 100)
(midi-wait m 2 120)
(midi-n-on m 2 1 (midi-symbolic '(a 3)) 0)
(midi-n-on m 2 1 (midi-symbolic '(g 4)) 0)
(midi-n-on m 2 1 (midi-symbolic '(a 4)) 0)

(midi-wait m 1 18) ;; Wait for left hand
(midi-n-on m 1 0 (midi-symbolic '(c sh 6)) 80) (midi-wait m 1 5)
(midi-n-on m 1 0 (midi-symbolic '(e 6)) 80) (midi-wait m 1 5)
(midi-n-on m 1 0 (midi-symbolic '(g 6)) 80) (midi-wait m 1 5)
(midi-n-on m 1 0 (midi-symbolic '(b fl 5)) 80)
(midi-wait m 1 27)
(midi-n-on m 1 0 (midi-symbolic '(c sh 6)) 0)
(midi-n-on m 1 0 (midi-symbolic '(e 6)) 0)
(midi-n-on m 1 0 (midi-symbolic '(g 6)) 0)
(midi-n-on m 1 0 (midi-symbolic '(b fl 5)) 0)
(midi-wait m 1 60)
(midi-wait m 1 18) ;; Wait for left hand
(midi-n-on m 1 0 (midi-symbolic '(d 6)) 80) (midi-wait m 1 5)
(midi-n-on m 1 0 (midi-symbolic '(f na 6)) 80) (midi-wait m 1 5)
(midi-n-on m 1 0 (midi-symbolic '(g sh 6)) 80) (midi-wait m 1 5)
(midi-n-on m 1 0 (midi-symbolic '(b 6)) 80)
(midi-wait m 1 27)
(midi-n-on m 1 0 (midi-symbolic '(d 6)) 0)
(midi-n-on m 1 0 (midi-symbolic '(f na 6)) 0)
(midi-n-on m 1 0 (midi-symbolic '(g sh 6)) 0)
(midi-n-on m 1 0 (midi-symbolic '(b 6)) 0)
(midi-wait m 1 60)

(midi-n-on m 1 0 (midi-symbolic '(e 5)) 100)
(midi-n-on m 1 0 (midi-symbolic '(a 5)) 100)
(midi-n-on m 1 0 (midi-symbolic '(d 6)) 100)
(midi-wait m 1 120)
(midi-n-on m 1 0 (midi-symbolic '(d 6)) 0)
(midi-n-on m 1 0 (midi-symbolic '(c sh 6)) 100)
(midi-wait m 1 120)
(midi-n-on m 1 0 (midi-symbolic '(e 5)) 0)
(midi-n-on m 1 0 (midi-symbolic '(a 5)) 0)
(midi-n-on m 1 0 (midi-symbolic '(c sh 6)) 0)

;(midi-sync m)  ;; Should be no-op

;; --- Bar 34 ---

(midi-n-on m 2 1 (midi-symbolic '(d 4)) 100)
(midi-n-on m 2 1 (midi-symbolic '(f sh 4)) 100)
(midi-n-on m 2 1 (midi-symbolic '(a 4)) 100)
(midi-wait m 2 480)
(midi-n-on m 2 1 (midi-symbolic '(d 4)) 0)
(midi-n-on m 2 1 (midi-symbolic '(f sh 4)) 0)
(midi-n-on m 2 1 (midi-symbolic '(a 4)) 0)

(midi-n-on m 1 0 (midi-symbolic '(f sh 5)) 100)
(midi-n-on m 1 0 (midi-symbolic '(a 5)) 100)
(midi-n-on m 1 0 (midi-symbolic '(d 6)) 100)
(midi-wait m 1 480)
(midi-n-on m 1 0 (midi-symbolic '(f sh 5)) 0)
(midi-n-on m 1 0 (midi-symbolic '(a 5)) 0)
(midi-n-on m 1 0 (midi-symbolic '(d 6)) 0)

;(midi-sync m)  ;; Should be no-op

(midi-finish m)
(midi-dump m (current-output-port))
